# vi: ts=4 expandtab
#
#    Copyright (C) 2012 Canonical Ltd.
#    Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
#    Copyright (C) 2012 Yahoo! Inc.
#
#    Author: Scott Moser <scott.moser@canonical.com>
#    Author: Juerg Haefliger <juerg.haefliger@hp.com>
#    Author: Joshua Harlow <harlowja@yahoo-inc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import glob
import importlib
import os
import sys


def import_module(module_name):
    __import__(module_name)
    return sys.modules[module_name]


def find_module(base_name, search_paths, required_attrs=None):
    if not required_attrs:
        required_attrs = []
    # NOTE(harlowja): translate the search paths to include the base name.
    lookup_paths = []
    for path in search_paths:
        real_path = []
        if path:
            real_path.extend(path.split("."))
        real_path.append(base_name)
        full_path = '.'.join(real_path)
        lookup_paths.append(full_path)
    found_paths = []
    for full_path in lookup_paths:
        mod = None
        try:
            mod = import_module(full_path)
        except ImportError:
            pass
        if not mod:
            continue
        found_attrs = 0
        for attr in required_attrs:
            if hasattr(mod, attr):
                found_attrs += 1
        if found_attrs == len(required_attrs):
            found_paths.append(full_path)
    return (found_paths, lookup_paths)


def search(dirs, checker=None):
    if checker is None:
        def checker(d):
            return True

    found = []
    for d in dirs:
        fdir = os.path.abspath(d)
        for f in glob.glob(os.path.join(fdir, "*.py")):
            if not os.path.isfile(f):
                continue
            bname = os.path.basename(f)
            if bname.startswith("__"):
                continue
            found.append((fdir, os.path.basename(f).partition(".")[0]))

        for init_py in glob.glob(os.path.join(fdir, "*", "__init__.py")):
            if not os.path.isfile(init_py):
                continue
            dirname = os.path.dirname(init_py)
            modname = dirname.rpartition(os.path.sep)[2]
            found.append((fdir, modname))

    modules = []
    for path, name in found:
        sys.path.insert(0, path)
        try:
            mod = importlib.import_module(name)
            if checker(mod):
                modules.append(mod)
        except Exception as e:
            print("WARN: failed loading %s" % name)
        sys.path.pop(0)

    return modules


def check_datasource_module(mod):
    # verify that a module is a datasource provider
    return callable(getattr(mod, 'get_datasources', None))


def load_datasource_modules(paths, checker=check_datasource_module):
    # search provided paths for datasource modules, return a list of modules
    return search(paths, checker=checker)


def get_datasources(paths, checker=check_datasource_module):
    # Return a sorted list of DataSource Classes that are found under paths
    modules = load_datasource_modules(paths, checker=checker)
    sources = []
    for m in modules:
        for ds in m.get_datasources():
            sources.append(ds)

    def _key(f):
        return f.get('priority')

    return sorted(sources, key=_key)

# vi: ts=4 expandtab
